package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
import org.openscm.kundo.plugins.context.* 
import org.openscm.kundo.common.logging.SimpleLogger

/**
 * Deploy application to running instance of Apache Tomcat server
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: Deploy application to running instance of Apache Tomcat server
 * 
 * <p> The delegate handles server deployment/redeployment operation on Apache Tomcat server
 * </p>
 */
/* 
 * @ant-target doDeployApplication
 */


class DeployAppDelegate extends AbstractProvisionDelegate {

	private def log
	private deployableFile
	/*
	 * Application Context path. This should be in the form of "/YourApplicationContextPath"
	 * @ant-property-name tomcat.application.context.path
	 */	
	protected String applicationContextPath	

	/*
	 * Application Location. This is the directory containing the application archive to be deployed
	 * @ant-property-name tomcat.application.location
	 */	
	protected String applicationLocation
	
	/*
	 * Application module prefix. defauls to file:// 
	 * @ant-property-name tomcat.application.prefix
	 */	
	protected String applicationPrefix
	
	/*
	 * Server Update. Defaults to true meaning it will reload the application if you are trying to deploy an already existing application
	 * @ant-property-name tomcat.application.update
	 */	
	protected String applicationUpdate


	/*
	 * Application Name. Name of the module to be deployed. This should include the file extension.
	 */	
	protected String applicationName


	

	/**
	* Injector for applicationContextPath element
	*/
	void setApplicationContextPath( String applicationContextPath ){
		this.applicationContextPath = applicationContextPath
	}	

	/**
	* Injector for applicationLocation element
	*/
	void setApplicationLocation( String applicationLocation ){
		this.applicationLocation = applicationLocation
	}	
	
	/**
	* Injector for applicationPrefix element
	*/
	void setApplicationPrefix( String applicationPrefix ){
		this.applicationPrefix = applicationPrefix
	}	
	
	/**
	* Injector for applicationName element
	*/
	void setApplicationName( String applicationName ){
		this.applicationName = applicationName
	}	

	/**
	* Injector for applicationUpdate element
	*/
	void setApplicationUpdate( String applicationUpdate ){
		this.applicationUpdate = applicationUpdate
	}	

	void doCheck(){
		deployableFile = this.applicationPrefix+this.applicationLocation+this.applicationName
		assert serverName != null
		assert serverPort != null
		assert serverManagerUrl != null
		assert applicationContextPath != null
		assert applicationLocation != null
		assert applicationPrefix != null
		assert deployableFile != null
		assert applicationName != null
		assert applicationUpdate != null
		assert !applicationContextPath.equals(""), "Application Context Path is empty"
		//assert applicationContextPath.charAt(0) == '/', "Application Context Path should beging with a /"
		assert !applicationName.equals(""), "Application Name is empty"
	}
	
	 /**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 DeployAppDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
		this.log = SimpleLogger.getInstance( ant )
	 }
	
	/* Concrete implementation of Server Deploy task */
	void doExecute(){
		ant.taskdef(resource:"org/apache/catalina/ant/antlib.xml", classpathref:"kundo-tomcat-plugin.dependency.path") 
		boolean isServerRunning = getServerStatus() 
		if(isServerRunning) {
			ant.deploy(	url:serverManagerUrl, 
						username:serverUserName, 
						password:serverUserPassword, 
						update:applicationUpdate, 
						path:applicationContextPath, 
						localWar:deployableFile)

			// Sleep for a while before moving to next task
			ant.sleep(seconds:serverSleep)
		} else {
			log.info("Cannot Deploy an application to a server thats in stopped status")
		}
	}

}