package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
 
/**
 * Perform start operation on the Apache Tomcat server
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: Starts server instance of Apache Tomcat server 
 * 
 * <p> The delegate handles server start operation of Apache Tomcat server
 * </p>
 */
/* 
 * @ant-target doStartServer
 */
 
class StartServerDelegate extends AbstractProvisionDelegate {
	 
	private def log
	private operatingSystem

	/*
	 * Command argument to be passed to ant exec task for starting the server
	 * @ant-property-name tomcat.server.start.cmd
	 */	
	protected String serverStartCmd
	/*
	 * Command argument to be passed to ant exec task for starting the server
	 * @ant-property-name tomcat.server.start.cmd.windows
	 */	
	protected String serverStartCmdWindows
	/*
	 * Command argument to be passed to ant exec task for starting the server
	 * @ant-property-name tomcat.server.start.cmd.other
	 */	
	protected String serverStartCmdOthers
	/*
	 * Value of Executable command for Windows
	 * @ant-property-name tomcat.server.executable.windows
	 */	
	protected String serverExecutableWindows
	/*
	 * Value of Executable command for Linux/Unix
	 * @ant-property-name tomcat.server.executable.others
	 */	
	protected String serverExecutableOthers
	
	 
	/**
	* Injector for StartCmd element
	* @ant-property-name tomcat.server.start.cmd
	*/
	void setServerStartCmd( String serverStartCmd ){
		this.serverStartCmd = serverStartCmd
	}	
	
	/**
	* Injector for StartCmdWindows element
	* @ant-property-name tomcat.server.start.cmd.windows
	*/
	void setServerStartCmdWindows( String serverStartCmdWindows ){
		this.serverStartCmdWindows = serverStartCmdWindows
	}	
	
	/**
	* Injector for StartCmdOthers element
	* @ant-property-name tomcat.server.start.cmd.others
	*/
	void setServerStartCmdOthers( String serverStartCmdOthers ){
		this.serverStartCmdOthers = serverStartCmdOthers
	}	
	
	/**
	* Injector for serverExecutableWindows element
	* @ant-property-name tomcat.server.executable.windows
	*/
	void setServerExecutableWindows( String serverExecutableWindows ){
		this.serverExecutableWindows = serverExecutableWindows
	}	
	
	/**
	* Injector for serverExecutableOthers element
	* @ant-property-name tomcat.server.executable.others
	*/
	void setServerExecutableOthers( String serverExecutableOthers ){
		this.serverExecutableOthers = serverExecutableOthers
	}	
	
	
	
	void doCheck(){
		// Check the operating system and assign the command appropriately
		operatingSystem = System.getProperty("os.name")
		assert operatingSystem != null
		if(operatingSystem.indexOf("Windows")!=-1) {
			log.debug("Operating system is Windows. Hence assigning the batch scripts")
			this.serverStartCmd = serverStartCmdWindows
			this.serverExecutable = serverExecutableWindows
		} else {
			log.debug("Operating system is Non Windows. Hence assigning the shell scripts")
			this.serverStartCmd = serverStartCmdOthers
			this.serverExecutable = serverExecutableOthers
		}
		assert serverStartCmd != null
		assert serverExecutable != null
		assert serverSpawn != null
		assert serverName != null
		assert serverPort != null
		assert serverScriptsDir != null
		assert serverHome != null
		assert !serverStartCmd.equals(""), "Server start command is empty"
	}
	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	StartServerDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
		this.log = SimpleLogger.getInstance( ant )
	}
	
	
	/* Concrete implementation of Server stop task */
	void doExecute() {
		// check the server status before stopping
		boolean isServerRunning = getServerStatus()
		if(!isServerRunning) {
			log.debug("Starting Apache Tomcat Server ....")
			// Starts server in a new window
			ant.exec(	dir:serverScriptsDir, 
						executable:serverExecutable,
						spawn:serverSpawn) 
						{
							ant.arg(value:serverStartCmd)
						}
			// Sleep for a while before moving to next task
			ant.sleep(seconds:serverSleep)
		} else {
			log.info("Apache Tomcat Server has already been started")
		}
	} 
	
 }
	

