package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
import org.openscm.kundo.plugins.context.* 
import org.openscm.kundo.common.logging.SimpleLogger

/**
 * UnDeploy application from a running instance of Apache Tomcat server
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: Deploy application to running instance of Apache Tomcat server
 * 
 * <p> The delegate handles server deployment/redeployment operation on Apache Tomcat server
 * </p>
 */
/* 
 * @ant-target doUnDeployApplication
 */


class UnDeployAppDelegate extends AbstractProvisionDelegate {

	private def log

	/*
	 * Application Context path. This should be in the form of "/YourApplicationContextPath"
	 * @ant-property-name tomcat.application.context.path
	 */	
	protected String applicationContextPath	

	/**
	* Injector for applicationContextPath element
	*/
	void setApplicationContextPath( String applicationContextPath ){
		this.applicationContextPath = applicationContextPath
	}	

	void doCheck(){
		assert applicationContextPath != null
		assert !applicationContextPath.equals(""), "Application Context Path is empty"
		assert serverName != null
		assert serverPort != null
		assert serverManagerUrl != null
	}
	
	/**
	* Constructor sets ant and buildContext instances in super class
	* @param ant			AntBuilder instance
	* @param buildContext	BuildContext instance
	*/
	UnDeployAppDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
		this.log = SimpleLogger.getInstance( ant )
	}
	
	/* Concrete implementation of Server Deploy task */
	void doExecute(){
		ant.taskdef(resource:"org/apache/catalina/ant/antlib.xml", classpathref:"kundo-tomcat-plugin.dependency.path") 
		boolean isServerRunning = getServerStatus() 
		if(isServerRunning) {
			ant.undeploy(	url:serverManagerUrl, 
							username:serverUserName, 
							password:serverUserPassword, 
							path:applicationContextPath)

			// Sleep for a while before moving to next task
			ant.sleep(seconds:serverSleep)
		} else {
			log.info("Cannot UnDeploy an application from a server thats in stopped status")
		}
	
	}

}